local AddOnName, Engine = ...
local AWF               = LibStub('AceAddon-3.0'):NewAddon(AddOnName, 'AceConsole-3.0', 'AceEvent-3.0')
local L                 = LibStub('AceLocale-3.0'):GetLocale(AddOnName)
local LDB               = LibStub('LibDataBroker-1.1', true)
local LDBIcon           = LibStub('LibDBIcon-1.0', true)
local LSM               = LibStub('LibSharedMedia-3.0')
local db

local options
local default           = {
  global                = {
    defaults            = {
      minimapButton     = {hide = false},
      scanMessage       = false,
      scanMessageList   = 'at my location, on me, weed on, anchor on',
      scanMessageSound  = 'None'
    }
  }
}

local addonHandler      = {}

function addonHandler:unitGroupLookup(unitName)
  unitName = unitName:gsub('-' .. GetRealmName(),  '')

  if unitName == UnitName('player') then
    return 'player'
  end

  if IsInRaid() then
    for i = 1, MAX_RAID_MEMBERS do
      local raidName = GetRaidRosterInfo(i)

      if raidName ~= nil and raidName == unitName then
        return 'Raid' .. i
      end
    end

    return nil
  end

  for i = 1, 4 do
    if UnitName('Party' .. i) ~= nil and UnitName('Party' .. i) == unitName then
      return 'Party' .. i
    end
  end

  return nil
end

function addonHandler:unitWorldPosition(unitLookup)
  local unitMapID     = C_Map.GetBestMapForUnit(unitLookup)
  local unitX, unitY  = C_Map.GetPlayerMapPosition(unitMapID, unitLookup):GetXY()
  return unitMapID, unitX, unitY
end

function addonHandler:stringSplit(s, d)
  r = {}
  for m in (s .. d):gmatch('(.-)' .. d) do
    tinsert(r, m)
  end
  return r
end

function AWF:OnInitialize()
  GetItemInfo(152510) -- cache anchor weed item?!?!? allows to not get an error when calling /awf later on

  -- register database
  self.db = LibStub('AceDB-3.0'):New(AddOnName .. 'DB', default)
  db = self.db.global.defaults

  -- register options table
  LibStub('AceConfigRegistry-3.0'):RegisterOptionsTable(AddOnName, options)
	self.BlizzardOptions = LibStub('AceConfigDialog-3.0'):AddToBlizOptions(AddOnName, L['AWF'])

  -- register slash command
  self:RegisterChatCommand(L['AWF_SLASHCMD'], function(...)
    local paramOne = ...

    if not paramOne or paramOne:trim() == '' then
      if not IsInGroup() then
        DEFAULT_CHAT_FRAME:AddMessage(L['AWF_MESSAGE_NOTINGROUP'])
        return
      end

      local awChannel       = IsInRaid() and ((UnitIsGroupAssistant('player') or UnitIsGroupLeader('player')) and 'RAID_WARNING' or 'RAID') or 'PARTY'
      local _, awItemLink   = GetItemInfo(152510)
      local _, unitX, unitY = addonHandler:unitWorldPosition('player')
      local awMessage       = format(L['AWF_MESSAGE_ANCHORWEED_FOUND'], awItemLink, tonumber(format('%.2f', unitX * 100)), tonumber(format('%.2f', unitY * 100)), UnitName('player'))

      SendChatMessage(awMessage, awChannel)
      return
    end

    if paramOne:trim() == 'config' then
      if InCombatLockdown() then
        DEFAULT_CHAT_FRAME:AddMessage(L['AWF_MESSAGE_INCOMBATLOCKDOWN'])
        return
      end

      InterfaceOptionsFrame_OpenToCategory(L['AWF'])
      InterfaceOptionsFrame_OpenToCategory(L['AWF'])
      return
    end
  end)

  -- register databroker and minimap icon
  local AWFLauncher = LDB:NewDataObject(AddOnName, {
    type    = 'launcher',
    icon    = 'interface/icons/inv_misc_herb_anchorweed',
    OnClick = function(self, button)
      if button == 'LeftButton' then
        db.scanMessage = not db.scanMessage
        if not db.scanMessage then
          DEFAULT_CHAT_FRAME:AddMessage(L['AWF_MESSAGE_CHATSCANNING_OFF'])
        else
          DEFAULT_CHAT_FRAME:AddMessage(format(L['AWF_MESSAGE_CHATSCANNING'], db.scanMessageList))
        end
        return
      else
        if InCombatLockdown() then
          DEFAULT_CHAT_FRAME:AddMessage(L['AWF_MESSAGE_INCOMBATLOCKDOWN'])
          return
        end

        InterfaceOptionsFrame_OpenToCategory(L['AWF'])
        InterfaceOptionsFrame_OpenToCategory(L['AWF'])
        return
      end
    end,
    OnTooltipShow = function(tooltip)
      tooltip:AddLine(L['AWF'])
      tooltip:AddLine('\n\r' .. L['AWF_ICON_CLICK_LEFT'])
      tooltip:AddLine(L['AWF_ICON_CLICK_RIGHT'])
    end
  })
  LDBIcon:Register(AddOnName .. '_MinimapButton', AWFLauncher, {hide = not db.minimapButton.hide})

  -- register events
  self:RegisterEvent('CHAT_MSG_RAID', 'ScanChatChannel')
  self:RegisterEvent('CHAT_MSG_RAID_WARNING', 'ScanChatChannel')
  self:RegisterEvent('CHAT_MSG_PARTY', 'ScanChatChannel')
  self:RegisterEvent('CHAT_MSG_PARTY_LEADER', 'ScanChatChannel')
end

function AWF:ScanChatChannel(...)
  if not db.scanMessage or db.scanMessageList:trim() == '' then
    return
  end

  local _, uMessage, uSender = ...

  for _, v in pairs(addonHandler:stringSplit(db.scanMessageList, ',')) do
    if strfind(strlower(uMessage), v:trim()) ~= nil then
      PlaySoundFile(LSM:Fetch('sound', db.scanMessageSound))

      if IsAddOnLoaded('TomTom') then
        local unitM, unitX, unitY = addonHandler:unitWorldPosition(addonHandler:unitGroupLookup(uSender))
        TomTom:AddWaypoint(unitM, unitX, unitY, {title = format(L['AWF_MESSAGE_TOMTOMTITLE'], date('%H:%M:%S'), uSender), from = uSender})
      end

      return
    end
  end
end

options                 = {
  type                  = 'group',
  name                  = L['AWF'],
  args                  = {
    minimapButton       = {
      type              = 'toggle',
      name              = L['AWF_OPTIONS_MINIMAPICON'],
      desc              = L['AWF_OPTIONS_MINIMAPICON_DESC'],
      width             = 'full',
      get               = function() return db.minimapButton.hide end,
      set               = function()
        db.minimapButton.hide = not db.minimapButton.hide
        LDBIcon:Show(AddOnName .. '_MinimapButton')

        if not db.minimapButton.hide then
          LDBIcon:Hide(AddOnName .. '_MinimapButton')
        end
      end,
      order             = 1
    },
    scanMessage         = {
      type              = 'toggle',
      name              = L['AWF_OPTIONS_CHATSCANNING'],
      desc              = L['AWF_OPTIONS_CHATSCANNING_DESC'],
      get               = function() return db.scanMessage end,
      set               = function()
        db.scanMessage = not db.scanMessage
        if not db.scanMessage then
          DEFAULT_CHAT_FRAME:AddMessage(L['AWF_MESSAGE_CHATSCANNING_OFF'])
        else
          DEFAULT_CHAT_FRAME:AddMessage(format(L['AWF_MESSAGE_CHATSCANNING'], db.scanMessageList))
        end
      end,
      order             = 2
    },
    scanMessageInfo     = {
      type              = 'description',
      name              = L['AWF_OPTIONS_CHATSCANNING_INFO'],
      width             = 'double',
      order             = 3
    },
    scanMessageList     = {
      type              = 'input',
      name              = L['AWF_OPTIONS_CHATSCANLIST'],
      desc              = L['AWF_OPTIONS_CHATSCANLIST_DESC'],
      width             = 'full',
      get               = function() return db.scanMessageList end,
      set               = function(_, value) db.scanMessageList = value end,
      order             = 4
    },
    scanMessageSound    = {
      type              = 'select',
      name              = L['AWF_OPTIONS_CHATSCANSOUND'],
      desc              = L['AWF_OPTIONS_CHATSCANSOUND_DESC'],
      dialogControl     = 'LSM30_Sound',
      values            = AceGUIWidgetLSMlists.sound,
      get               = function() return  db.scanMessageSound end,
      set               = function(_, value)  db.scanMessageSound = value end,
      order             = 5
    }
  },
  order                 = 1
}
